==> kom%2B%2B-0.3.tar.gz: kom++-0.3/libkom++/pers-cache.cc <==
// Fetch-on-demand cache of persons.
//
// $Id: pers-cache.cc,v 1.7 1994/08/01 21:37:05 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <stdlib.h>
#include "pers-cache.h"
#include "question.h"
#include "misc.h"
#include "error-codes.h"

static Pers_stat bad_pers_stat;
static Pers_stat zero_pers_stat(KOM_CONF_ZERO);

Pers_cache::Pers_cache(Connection *c)
{
    conn = c;
}

Pers_cache::Pers_cache(const Pers_cache& c)
{
    *this = c;
}

Pers_cache &
Pers_cache::operator=(const Pers_cache& c)
{
    abort();
    return *this;
}

Pers_cache::~Pers_cache()
{
}

Bool
Pers_cache::present(Pers_no cno) const
{
    return statusmap.present(cno);
}

//void
//Pers_cache::clear()
//{
//    cerr << "Unimp\n";
//    abort();
//}


Status
Pers_cache::prefetch(Pers_no cno)
{
    if (cno <= 0)
	return st_error;

    if (statusmap.present(cno))
    {
	if (statusmap[cno].valid())
	    return st_ok;
	else
	    return st_error;
    }
    else
    {
	if (!pendingmap.present(cno))
	{
	    pendingmap[cno] = new get_person_stat_question(conn, cno);
	    return st_pending;
	}
	else
	{
	    switch (pendingmap[cno]->status())
	    {
	    case st_pending:
		return st_pending;
	    case st_ok:
		statusmap[cno] = pendingmap[cno]->result();
		pendingmap.delete_entry(cno);
		return st_ok;
	    case st_error:
		statusmap[cno] = Pers_stat(pendingmap[cno]->error());
		pendingmap.delete_entry(cno);
		return st_error;
	    default:
		abort();
	    }
	}
    }
    abort();
}


Pers_stat
Pers_cache::get(Pers_no cno)
{
    if (cno <= 0)
	return zero_pers_stat;

    if (statusmap.present(cno))
	return statusmap[cno];
    else if (pendingmap.present(cno))
    {
	get_person_stat_question *gtr = pendingmap[cno];
	if (gtr->receive() == st_ok)
	    statusmap[cno] = gtr->result();
	else
	    statusmap[cno] = Pers_stat(gtr->error());
	pendingmap.delete_entry(cno);
	return statusmap[cno];
    }
    else		
    {
	// Not present in cache, and no outstanding question.
	get_person_stat_question gt(conn, cno);
	if (gt.receive() == st_ok)
	    statusmap[cno] = gt.result();
	else
	    statusmap[cno] = Pers_stat(gt.error());
	return statusmap[cno];
    }
    abort();
}

//void
//Pers_cache::remove(Pers_no cno)
//{
//    abort();
//}

Bool
Pers_cache::Ppmap::present(Pers_no c) const
{
    // contains() really should be a const member function. This is a 
    // workaround. The line below should be substituted.
    return ((Pers_cache::Ppmap*)this)->contains(c) ? TRUE : FALSE;
    // return contains(c) ? TRUE : FALSE;
}

Pers_stat &
Pers_cache::Ppmap::operator[](Pers_no c)
{
    return Pers_noPers_statCHMap::operator[](c);
}


Pers_cache::Ppmap::Ppmap()
    : Pers_noPers_statCHMap(bad_pers_stat, 50)
{
}

Bool
Pers_cache::Ppreqmap::present(Pers_no c)
{
    return contains(c) ? TRUE : FALSE;
}

void
Pers_cache::Ppreqmap::delete_entry(Pers_no c)
{
    delete operator[](c);
    del(c);
}

get_person_stat_question *&
Pers_cache::Ppreqmap::operator[](Pers_no c)
{
    return Pers_noget_person_stat_question_ptrCHMap::operator[](c);
}

static get_person_stat_question_ptr bad_ptr = NULL;

Pers_cache::Ppreqmap::Ppreqmap()
    : Pers_noget_person_stat_question_ptrCHMap(bad_ptr)
{
}

Pers_cache::Ppreqmap::~Ppreqmap()
{
    for (Pix i = first(); i != 0; next(i))
    {
	delete contents(i);
    }
}

==> kom%2B%2B-0.5.tar.gz: kom++-0.5/libkom++/pers-cache.cc <==
// Fetch-on-demand cache of persons.
//
// $Id: pers-cache.cc,v 1.11 1995/11/18 22:49:50 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <stdlib.h>
#include "pers-cache.h"
#include "question.h"
#include "misc.h"
#include "error-codes.h"

static Pers_stat bad_pers_stat;
static Pers_stat zero_pers_stat(KOM_CONF_ZERO);

Pers_cache::Pers_cache(Connection *c)
{
    conn = c;
}

Pers_cache::~Pers_cache()
{
}

bool
Pers_cache::present(Pers_no cno) const
{
    return statusmap.present(cno);
}

//void
//Pers_cache::clear()
//{
//    cerr << "Unimp\n";
//    abort();
//}


Status
Pers_cache::prefetch(Pers_no pno)
{
    if (pno <= 0)
	return st_error;

    if (statusmap.present(pno))
    {
	if (statusmap[pno].errno() == KOM_NO_ERROR)
	    return st_ok;
	else
	    return st_error;
    }
    else
    {
	if (!pendingmap.present(pno))
	{
	    pendingmap[pno] = new get_person_stat_question(conn, pno);
	    return st_pending;
	}
	else
	{
	    switch (pendingmap[pno]->status())
	    {
	    case st_pending:
		return st_pending;
	    case st_ok:
		statusmap[pno] = pendingmap[pno]->result();
		pendingmap.delete_entry(pno);
		return st_ok;
	    case st_error:
		statusmap[pno] = Pers_stat(pendingmap[pno]->error());
		pendingmap.delete_entry(pno);
		return st_error;
	    }
	}
    }
    abort();
}


Pers_stat
Pers_cache::get(Pers_no pno)
{
    if (pno <= 0)
	return zero_pers_stat;

    if (statusmap.present(pno))
	return statusmap[pno];
    else if (pendingmap.present(pno))
    {
	Pers_stat res;
	get_person_stat_question *gtr = pendingmap[pno];
	if (gtr->receive() == st_ok)
	    res = gtr->result();
	else
	    res = Pers_stat(gtr->error());
	statusmap[pno] = res; 
	pendingmap.delete_entry(pno);
	return res;
    }
    else		
    {
	// Not present in cache, and no outstanding question.
	Pers_stat res;
	get_person_stat_question gt(conn, pno);
	if (gt.receive() == st_ok)
	    res = gt.result();
	else
	    res = Pers_stat(gt.error());
	statusmap[pno] = res;
	return res;
    }
    abort();
}

//void
//Pers_cache::remove(Pers_no pno)
//{
//    abort();
//}

bool
Pers_cache::Ppmap::present(Pers_no c) const
{
    // contains() really should be a const member function. This is a 
    // workaround. The line below should be substituted.
    return ((Pers_cache::Ppmap*)this)->contains(c) ? true : false;
    // return contains(c) ? true : false;
}

Pers_stat &
Pers_cache::Ppmap::operator[](Pers_no c)
{
    return Pers_noPers_statCHMap::operator[](c);
}


Pers_cache::Ppmap::Ppmap()
    : Pers_noPers_statCHMap(bad_pers_stat, 50)
{
}

bool
Pers_cache::Ppreqmap::present(Pers_no c)
{
    return contains(c) ? true : false;
}

void
Pers_cache::Ppreqmap::delete_entry(Pers_no c)
{
    delete operator[](c);
    del(c);
}

get_person_stat_question *&
Pers_cache::Ppreqmap::operator[](Pers_no c)
{
    return Pers_noget_person_stat_question_ptrCHMap::operator[](c);
}

static get_person_stat_question_ptr bad_ptr = NULL;

Pers_cache::Ppreqmap::Ppreqmap()
    : Pers_noget_person_stat_question_ptrCHMap(bad_ptr)
{
}

Pers_cache::Ppreqmap::~Ppreqmap()
{
    for (Pix i = first(); i != 0; next(i))
    {
	delete contents(i);
    }
}
